#//-----------------------------------------------
#//TUSB-S01VIN1用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_07_24　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#Device DLL Loader ( cdecl )
tusbs01vin1dll = cdll.LoadLibrary("TUSBSVI1.DLL")

#######  dll function definitions (do not call directry from user codes)
tusbs01vin1dll.Tusbs01vin1_Device_Open.restype = c_short
tusbs01vin1dll.Tusbs01vin1_Device_Open.argtypes = (c_short,)
tusbs01vin1dll.Tusbs01vin1_Device_Close.restype = None
tusbs01vin1dll.Tusbs01vin1_Device_Close.argtypes = (c_short,)
tusbs01vin1dll.Tusbs01vin1_Timer_Set.restype = c_short
tusbs01vin1dll.Tusbs01vin1_Timer_Set.argtypes = (c_short,c_ubyte,c_uint)
tusbs01vin1dll.Tusbs01vin1_Input_Set.restype = c_short
tusbs01vin1dll.Tusbs01vin1_Input_Set.argtypes = (c_short,c_ubyte)
tusbs01vin1dll.Tusbs01vin1_Sampling_Start.restype = c_short
tusbs01vin1dll.Tusbs01vin1_Sampling_Start.argtypes = (c_short,c_ubyte,c_short,c_uint,c_uint)
tusbs01vin1dll.Tusbs01vin1_Status_Read.restype = c_short
tusbs01vin1dll.Tusbs01vin1_Status_Read.argtypes = (c_short,POINTER(c_ubyte),POINTER(c_ubyte),POINTER(c_uint))
tusbs01vin1dll.Tusbs01vin1_Memory_Read.restype = c_short
tusbs01vin1dll.Tusbs01vin1_Memory_Read.argtypes = (c_short,POINTER(c_short),POINTER(c_uint))
tusbs01vin1dll.Tusbs01vin1_Sampling_Stop.restype = c_short
tusbs01vin1dll.Tusbs01vin1_Sampling_Stop.argtypes = (c_short,)
tusbs01vin1dll.Tusbs01vin1_Memory_Clear.restype = c_short
tusbs01vin1dll.Tusbs01vin1_Memory_Clear.argtypes = (c_short,)
tusbs01vin1dll.Tusbs01vin1_Sampling_Trigger.restype = c_short
tusbs01vin1dll.Tusbs01vin1_Sampling_Trigger.argtypes = (c_short,)
tusbs01vin1dll.Tusbs01vin1_FifoPeak.restype = c_short
tusbs01vin1dll.Tusbs01vin1_FifoPeak.argtypes = (c_short,POINTER(c_ubyte))

#######  support functions (do not call directry from user codes)
def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):
        return False
    elif(Val < Min):
        return False
    elif(Val > Max):
        return False
    else:
        return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

######## def functions   ###############################################

def Tusbs01vin1_Device_Open(DevId):
    """Open Device
    
    Arguments:
    DevId           ----    Device Id 0~15
    
    Returns:
    int             ----    Result code
    """
    if( IsValidId(DevId)):
        return tusbs01vin1dll.Tusbs01vin1_Device_Open(DevId)
    else:
        return 1

def Tusbs01vin1_Device_Close(DevId):
    """Close Device
    
    Arguments:
    DevId           ----    Device Id 0~15
    """
    if( IsValidId(DevId)):
        tusbs01vin1dll.Tusbs01vin1_Device_Close(DevId)


def Tusbs01vin1_Timer_Set(DevId,TimerN,TimerM):
    """Set Timer
    
    Arguments:
    DevId           ----    Device Id 0~15
    TimerN          ----    Timer N Value 0~4
    TimerM          ----    Timer M Value 1~65536
    
    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 1
    if not IsInRange(TimerN,0,4): return 8
    if not IsInRange(TimerM,1,65536): return 8
    return tusbs01vin1dll.Tusbs01vin1_Timer_Set(DevId,TimerN,TimerM)

def Tusbs01vin1_Input_Set(DevId,Range):
    """Set Input Settings
    
    Arguments:
    DevId           ----    Device Id 0~15
    HPF             ----    High pass filter 0:0.1Hz 1:1Hz 2:10Hz
    Range           ----    Range 0:x1 1:x10 2:x100

    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 1
    if not IsInRange(Range,0,2): return 8
    return tusbs01vin1dll.Tusbs01vin1_Input_Set(DevId,Range)

def Tusbs01vin1_Sampling_Start(DevId,TrgType,Level,PreLen,TotalLen):
    """Start Sampling
    
    Arguments:
    DevId           ----    Device Id 0~15
    TrgType         ----    Trigger Type 0~5
    Level           ----    Analog Trigger Level
    PreLen          ----    Pretrigger length 0~10,000,000
    TotalLen        ----    Driver buffer length ~10,000,000

    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 1
    if not IsInRange(TrgType,0,5): return 8
    if not IsInRange(PreLen,0,10000000): return 8
    if not IsInRange(TotalLen,0,10000000): return 8
    return tusbs01vin1dll.Tusbs01vin1_Sampling_Start(DevId,TrgType,Level,PreLen,TotalLen)

def Tusbs01vin1_Status_Read(DevId):
    """Read Sampling Status
    
    Arguments:
    DevId           ----    Device Id 0~15
    
    Rerutns:
    int,            ----    Result code
    int,            ----    Sampling Status
    int,            ----    Overflow Status
    int,            ----    Data Length in Sampling buffer
    """
    if not IsValidId(DevId): return 1,None,None,None
    SmplStat = c_ubyte()
    OvfStat = c_ubyte()
    DatLen = c_uint()
    ret = tusbs01vin1dll.Tusbs01vin1_Status_Read(DevId,byref(SmplStat),byref(OvfStat),byref(DatLen))
    if ret != 0: return ret,None,None,None
    return 0,SmplStat.value,OvfStat.value,DatLen.value

def Tusbs01vin1_Memory_Read(DevId,ReqLen):
    """Read Sampled Data
    
    Arguments:
    DevId           ----    Device Id 0~15
    ReqLen          ----    Request Data length

    Returns:
    int,            ----    Result code
    (int),          ----    Data list
    """
    if not IsValidId(DevId): return 1,None
    Data = ( c_short * ReqLen)()
    p_Data = cast(pointer(Data),POINTER(c_short))
    leng = c_uint(ReqLen)
    ret = tusbs01vin1dll.Tusbs01vin1_Memory_Read(DevId,p_Data,byref(leng))
    if ret != 0: return ret,None
    rData = [Data[i] for i in range(leng.value)]
    return 0,rData

def Tusbs01vin1_Sampling_Stop(DevId):
    """Stop Sampling
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 1
    return tusbs01vin1dll.Tusbs01vin1_Sampling_Stop(DevId)

def Tusbs01vin1_Memory_Clear(DevId):
    """Free Sampling Memory
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 1
    return tusbs01vin1dll.Tusbs01vin1_Memory_Clear(DevId)

def Tusbs01vin1_Sampling_Trigger(DevId):
    """Generate Sampling Trigger
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int             ----    Result code
    """
    if not IsValidId(DevId): return 1
    return tusbs01vin1dll.Tusbs01vin1_Sampling_Trigger(DevId)

def Tusbs01vin1_FifoPeak(DevId):
    """Read Fifo usage rate
    
    Arguments:
    DevId           ----    Device Id 0~15

    Returns:
    int,            ----    Result code
    int,            ----    rate (%)
    """
    if not IsValidId(DevId): return 1
    rate = c_ubyte()
    ret = tusbs01vin1dll.Tusbs01vin1_FifoPeak(DevId,byref(rate))
    if ret != 0: return ret,None
    return 0,rate.value

#戻り値用エラーメッセージ
ErrStrDic = {
            0:"正常終了しました",
            1:"ID番号が不正です",
            2:"ドライバがインストールされていません",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
            7:"オープンされていません",
            8:"パラメータエラー",
            9:"USB通信エラーです",
            10:"作業用メモリ領域が確保できません",
            11:"連続取り込み動作中です",
            12:"連続取り込みデータはありません",
            99:"その他のエラーです",
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"
